<?php
namespace Concrete\Package\OunziwNemqr\Block\OunziwNemqr;

defined('C5_EXECUTE') or die("Access Denied.");

use Concrete\Core\Block\BlockController;
use Concrete\Core\Package\Package;


class Controller extends BlockController
{
    protected $btTable = 'btOunziwNemqr';
    protected $btInterfaceWidth = "360";
    protected $btWrapperClass = 'ccm-ui';
    protected $btInterfaceHeight = "400";
    protected $btCacheBlockRecord = true;
    protected $btCacheBlockOutput = true;
    protected $btCacheBlockOutputOnPost = true;
    protected $btCacheBlockOutputForRegisteredUsers = true;

    public function getBlockTypeDescription()
    {
        return t('Creates NEM QR Code');
    }

    public function getBlockTypeName()
    {
        return t("NEM QR");
    }
    public function add()
    {
        $this->set('size', 200);
        $this->set('padding', 10);
    }

    public function view()
    {
        $pkg = Package::getByHandle('qr_code');
        if (is_object($pkg)) {
            $vendorPath = $pkg->getPackagePath() . '/vendor/';
            require_once $vendorPath . 'autoload.php';

            if ($this->validate_wallet($this->wallet) && is_numeric($this->amount) ) {
                $nemdata = json_encode(array(
                    'v' => 2,
                    'type' => 2,
                    'data' => array(
                        'addr' => str_replace('-','',$this->wallet),
                        'amount' => intval(1000000 * $this->amount),
                        'msg' => $this->message,
                        'name' => 'ounziw_nemqr',
                    )
                ));

                if (!is_numeric($this->size)) {
                    $this->size = 250;
                }
                if (!is_numeric($this->padding)) {
                    $this->padding = 10;
                }
                $qrCode = new \Endroid\QrCode\QrCode();
                $qrCode
                    ->setText($nemdata)
                    ->setSize($this->size)
                    ->setPadding($this->padding);

                $this->set('qrCode', $qrCode);

                $alt = $this->amount . ' XEM';
                $this->set('alt', $alt);
            } else {
                \Log::addEntry("Invalid Wallet ID or amount.");
            }
        } else {
            \Log::addEntry("Package qr_code seems not exist.");
        }
    }

    /**
     * @return Error
     */
    public function validate($args)
    {
        $e = $this->app->make('helper/validation/error');

        // amount: numeric
        if ( ! is_numeric($args['amount']) ) {
            $e->add(t('Invalid Amount.'));
        }
        // size: empty or numeric
        if ( !is_numeric($args['size']) ) {
            $e->add(t('Invalid Size.'));
        }
        // padding: empty or numeric
        if ( !is_numeric($args['padding']) ) {
            $e->add(t('Invalid Padding.'));
        }

        if (! $this->validate_wallet($args['wallet']) ) {
            $e->add(t('Invalid Address.') .t('Enter 40-digit numbers/capital-alphabets. (Address can be separated with "-".)'));
        }

        return $e;
    }
    public function validate_wallet($wallet) {

        $address_valid = false;
        // wallet address: 40 alphabets/numbers (capital alphabets)
        if ( strlen($wallet) === 40 && strlen($wallet) === strspn($wallet,'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ') ) {
            $address_valid = true;
        }
        // wallet address can be splitted by "-", 6-6-6-6-6-6-4
        if ( strlen($wallet) === 46 && strlen($wallet) === strspn($wallet,'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ-') ) {
            $address_valid = true;
        }
        return $address_valid;
    }

}